@@ -21,7 +21,6 @@ from . import (
     pysbuiltins,
     results,
     runner,
-    singletons,
     symtab,
     token,
     utils,
@@ -38,6 +37,7 @@ __all__ = (
     'exceptions',
     'handlers',
     'highlight',
+    'immutables',
     'interpreter',
     'lexer',
     'nodes',
